from __future__ import annotations

from .core import (
    OperaFunction,
    Operas,
    OperasRegistry,
    get_global_operas,
    get_global_operas_registry,
)

__all__ = [
    "OperaFunction",
    "OperasRegistry",
    "Operas",
    "get_global_operas_registry",
    "get_global_operas",
]
